/*-------------------------------------------------------------------*/
/*  Csum.h, By B. Mours Oc 30, 2000                                */
/*-------------------------------------------------------------------*/
#ifndef CSUMMARY
#define CSUMMARY

#ifndef _POSIX_SOURCE
#define _POSIX_SOURCE
#endif

#include "FrameL.h"
#include "Frv.h"

#ifdef __cplusplus
extern "C" {
#endif

struct Csum {
  FrVect *chi2;       /* vector to hold the pointer to the chi2 algorithms   */
  FrVect *index;      /* vector to hold the index to the chi2 algorithms     */
  FrVect *previous;   /* vector to hold the previous ADC values              */
  int    nCall;       /* nulmber of calls                                    */
  int    nDataMin;    /* minimum number of data point to use an ADC          */
  int    nDataMax;    /* maximum number of data point to use an ADC          */
  int    nDataSlow;   /* maximum sampling rate to the a slow channel         */
  unsigned int GTimeS;/* GPS time for the first processed frame              */
  double meanChi2;    /* mean chisquare value                                */
  double threshold;   /* threshold used to stored loudest segment info       */
                      /*    If = 0; no data are saved                        */
  char*  prefix;      /* prefix used for output names (H=Hanford, L=Livi,..) */

  int     resPeriod;  /* period used to compute the residuals                */
  double  resScale;   /* scalling factor used to compute the residuals       */
  FrvRes* meanRes;    /* algorithm to compute the mean residuals             */
  FrvRes* rmsRes;     /* algorithm to compute the rms  residuals             */
  FrvRes* pwr1Res;    /* algorithm to compute the power1 residuals           */
  FrvRes* pwr2Res;    /* algorithm to compute the power2 residuals           */
  FrvRes* pwr3Res;    /* algorithm to compute the power3 residuals           */
  FrvRes* deltaRes;   /* algorithm to compute the residuals for delta        */
  FrvRes* slowRes;    /* algorithm to compute the mean residuals for slow ch.*/
};
 
typedef struct Csum Csum;

Csum* CsumNew(int nDataMin, int nDataMax, double threshold, char * prefix);
int   CsumInit (Csum *cs, FrameH *frame);
int   CsumProc (Csum *cs, FrameH *frame);
int   CsumSlow (Csum *cs, FrameH *frame);
int   CsumSaveS(Csum *cs, FrameH *frame);

void  CsumDumpF(FrameH *frame, char* prefix, char* tag, FILE* out);
void  CsumDumpS(FrameH *frame, char* prefix, char* tag, FILE* out);


#ifdef __cplusplus
}
#endif

#endif
