////////////////////////////////////////////////////////////////////////////////
//                                                                            //
//                           ManagedSegmentList.hh                            //
//                                                                            //
//             Container for managing segment lists used by hMon              //
//                                                                            //
//                            Shourov K. Chatterji                            //
//                                                                            //
//                                2008-Sep-04                                 //
//                                                                            //
////////////////////////////////////////////////////////////////////////////////

// $Id: ManagedSegmentList.hh,v 1.1 2008/09/05 16:39:10 shourov Exp $

#ifndef MANAGEDSEGMENTLIST_HH
#define MANAGEDSEGMENTLIST_HH

#include "Time.hh"
#include "Interval.hh"
#include "SegList.hh"
#include <string>
#include <map>
#include <set>

class ManagedSegmentList
{

public:

  ManagedSegmentList();

  ~ManagedSegmentList();

  const std::string& getFileName(void) const;
  const std::string& getListName(void) const;
  const std::string& getListDescription(void) const;

  bool isDispatchMode(std::string mode);

  Interval getMaximumDuration(void) const;
  Interval getRecentDuration(void) const;

  const Time& getLastUpdateTime(void) const;

  const LockSegList& getWriteSegments(void) const;
  const LockSegList& getUnsentSegments(void) const;
  const LockSegList& getRecentSegments(void) const;

  void setFileName(const std::string& name);
  void setListName(const std::string& name);
  void setListDescription(const std::string& description);

  void addDispatchMode(std::string mode);
  void removeDispatchMode(std::string mode);
  void clearDispatchModes(void);

  void setMaximumDuration(const Interval& duration);
  void setRecentDuration(const Interval& duration);

  void insert(const LockSegment& segment);
  void insert(const LockSegList& segmentList);
  void insert(const Time& startTime, const Time& stopTime);
  void insert(const Time& startTime, const Interval& duration);

  void update(const Time& updateTime);

  void markSent(void);

  void reset(void);

  void mask(LockSegList& segmentList,
            const Time& startTime = Time(0),
            const Time& stopTime = Time(SEGMENT_TINF)) const;

private:

  mutable LockSegList writeSegments;
  mutable LockSegList unsentSegments;
  mutable LockSegList recentSegments;

  mutable bool isMerged;

  Time lastUpdateTime;

  Interval maximumDuration;
  Interval recentDuration;

  std::string fileName;
  std::string listName;
  std::string listDescription;
  std::set<std::string> dispatchModes;

  void merge(void) const;

};

typedef std::map<std::string, ManagedSegmentList> ManagedSegmentListMap;

typedef ManagedSegmentListMap::iterator ManagedSegmentListMapIterator;

#endif // MANAGEDSEGMENTLIST_HH
