#include <tid.h>

void Tid::SetParamsFromFile(FILE* in) {

  char buf[2048];

  unsigned int lnumb = 0;
  bool OSCDef = 0;
  bool chnOSC;

  for(;;) {
    *buf=0;
    fgets(buf, 2048, in);

    lnumb++;

    if(buf[0]!='#' && buf[0]!='\n' && buf[0]!=0) { // check for comments or empty lines
      
      char *ptr;
      bool knownOption = 0;

      // General Options

      if(strstr(buf,"Source") == buf) {
	knownOption = 1;
	ptr = strchr(buf,'=');
	char* ptr2 = strrchr(buf,'\n');
	if(ptr2 != NULL) *ptr2 = '\x0';
	sName = string(ptr+1);

	if(!DMTBackGround) {

	  TidMessage(string("Connecting to data source (") +
		     sName +
		     string(")."));
  
	  In = new Dacc(sName.c_str());
	  if(!In) throw TidError("FATAL:Can't connect to data server.");
	  In->setBuffer(5); // allow some buffering of input data
	}

      }

      if(strstr(buf,"OSCConfig") == buf) {
	knownOption = 1;
	ptr = strchr(buf,'=');
	char* ptr2 = strrchr(buf,'\n');
	if(ptr2 != NULL) *ptr2 = '\x0';
	if(!In) throw TidError("FATAL:Must define the Source before OSCConfig.");
	osclist = new OperStateCondList(*In, 0);
	osclist->readConfig(ptr+1);
	OSCDef = 1;
      }

      if(strstr(buf,"OSCCondition") == buf) {
	knownOption = 1;
	ptr = strchr(buf,'=');
	char* ptr2 = strrchr(buf,'\n');
	if(ptr2 != NULL) *ptr2 = '\x0';
	if(!OSCDef) TidWarning("OSC defined while OSCConfig is missing. Ignoring.");
	else oscConditions.push_back(string(ptr+1));
      }

      if(strstr(buf,"Port") == buf) {
	knownOption = 1;
	ptr = strchr(buf,'=');
	IO = new TidIO(atoi(ptr+1)); // launch data server
	IOin = new TidIO(1+atoi(ptr+1));
	broadcaster = new TidBroadcast(*IO);
      }

      if(strstr(buf,"maxClusterSize") == buf) {
	knownOption = 1;
	ptr = strchr(buf,'=');
	maxClusterSize = (unsigned int)atoi(ptr+1);
      }
      
      if(strstr(buf,"TResolution") == buf) {
	knownOption = 1;
	ptr = strchr(buf,'=');
	tres = atof(ptr+1);
      }


      if(feof(in)!=0) break;

      // EventLists
      
      bool Stop = 0;
      if(strstr(buf,"EventsList:") == buf) {

	bool doFilter = 0;
	vector<dComplex> poles, zeroes;
	vector<double> polesReal, polesImag, zeroesReal, zeroesImag;
	bool haveRealPoles = 0, haveImagPoles = 0, haveRealZeroes = 0, haveImagZeroes = 0;

	chnOSC = 0;

	knownOption = 1;
	bool Abort = 0, allOptions = 0;
	unsigned long long defOptions = 0;
	unsigned int sline = lnumb;

	string chnName;
	unsigned int sigma, dec, fs;
	vector<unsigned int> delta;
	float alpha, pthresh;
	double flow, fhigh, stabTol, stabTime;
	vector<unsigned int> testFreq;

	for(;;) {

	  fgets(buf, 2048, in);

	  lnumb++;

	  if(buf[0]!='#' && buf[0]!='\n') { 
      
	    char *ptr;
	    bool sOption = 0;

	    if(strstr(buf,"OSCCheck") == buf) {
	      sOption=1;
	      chnOSC=1;
	    }

	    if(strstr(buf,"Channel") == buf) {
	      sOption = 1;
	      ptr = strchr(buf,'=');
	      char* ptr2 = strrchr(buf,'\n');
	      if(ptr2 != NULL) *ptr2 = '\x0';
	      chnName = string(ptr+1);
	      fs = GetSamplingByName(chnName);
	      defOptions |= 1<<0;
	    }

	    if(strstr(buf,"Sigma") == buf) {
	      sOption = 1;
	      ptr = strchr(buf,'=');
	      sigma = (unsigned int)atoi(ptr+1);

	      defOptions |= 1<<1;
	    }

	    if(strstr(buf,"Alpha") == buf) {
	      sOption = 1;
	      ptr = strchr(buf,'=');
	      alpha = atof(ptr+1);

	      defOptions |= 1<<2;
	    }

	    if(strstr(buf,"BPP") == buf) {
	      sOption = 1;
	      ptr = strchr(buf,'=');
	      pthresh = atof(ptr+1);

	      defOptions |= 1<<3;
	    }

	    if(strstr(buf,"LowFreq") == buf) {
	      sOption = 1;
	      ptr = strchr(buf,'=');
	      flow = atof(ptr+1);

	      defOptions |= 1<<5;
	    }

	    if(strstr(buf,"HighFreq") == buf) {
	      sOption = 1;
	      ptr = strchr(buf,'=');

	      if(strstr(ptr+1,"Nyquist") == ptr+1) {
		fhigh = GetSamplingByName(chnName)/2;
		dec = 1;
	      }
	      else {
		fhigh = atof(ptr+1);
		dec = (unsigned int)floor(GetSamplingByName(chnName)/fhigh);
	      }
	      defOptions |= 1<<6;
	    }

	    if(strstr(buf,"StabTimeScale") == buf) {
	      sOption = 1;
	      ptr = strchr(buf,'=');
	      stabTime = atof(ptr+1);

	      defOptions |= 1<<7;
	    }

	    if(strstr(buf,"StabTol") == buf) {
	      sOption = 1;
	      ptr = strchr(buf,'=');
	      stabTol = atof(ptr+1);

	      defOptions |= 1<<8;
	    }

	    if(strstr(buf,"StabTestFreq") == buf) {
	      sOption = 1;
	      defOptions |= 1<<9;

	      double BW = 1.0/tres; // bandwidth to convert to frequency index

	      char* ptr2;

	      ptr = strchr(buf,'='); 
	      ptr2 = strchr(buf,',');

	      if(strstr(ptr,"All")==ptr+1) {
		for(unsigned int i=1; i<(unsigned int)floor(fhigh/BW); i++)
		  testFreq.push_back(i);
	      }
	      else {

		while(ptr2 != NULL) {
		  *ptr2 = '\x0';
		  testFreq.push_back((unsigned int)floor(atoi(ptr+1)/BW));

		  ptr = ptr2;
		  ptr2 = strchr(ptr+1,',');
		}

		testFreq.push_back((unsigned int)floor(atoi(ptr+1)/BW));
	      }
	    }

	    if(strstr(buf,"Delta") == buf) {
	      sOption = 1;

	      defOptions |= 1<<4;


	      char* ptr2;

	      ptr = strchr(buf,'='); 
	      ptr2 = strchr(buf,',');

	      while(ptr2 != NULL) {
		*ptr2 = '\x0';
		delta.push_back((unsigned int)atoi(ptr+1));

		ptr = ptr2;
		ptr2 = strchr(ptr+1,',');
	      }

	      delta.push_back((unsigned int)atoi(ptr+1));
	    }




	    if(strstr(buf,"FilterPolesReal") == buf) {
	      sOption = 1;
	      haveRealPoles = 1;
	      doFilter = 1;

	      char* ptr2;

	      ptr = strchr(buf,'='); 
	      ptr2 = strchr(buf,',');

	      while(ptr2 != NULL) {
		*ptr2 = '\x0';
		polesReal.push_back(atof(ptr+1));

		ptr = ptr2;
		ptr2 = strchr(ptr+1,',');
	      }

	      polesReal.push_back(atof(ptr+1));	    
	    }

	    if(strstr(buf,"FilterPolesImag") == buf) {
	      sOption = 1;
	      haveImagPoles = 1;
	      doFilter = 1;

	      char* ptr2;

	      ptr = strchr(buf,'='); 
	      ptr2 = strchr(buf,',');

	      while(ptr2 != NULL) {
		*ptr2 = '\x0';
		polesImag.push_back(atof(ptr+1));

		ptr = ptr2;
		ptr2 = strchr(ptr+1,',');
	      }

	      polesImag.push_back(atof(ptr+1));	    
	    }

	    if(strstr(buf,"FilterZeroesReal") == buf) {
	      sOption = 1;
	      haveRealZeroes = 1;
	      doFilter = 1;

	      char* ptr2;

	      ptr = strchr(buf,'='); 
	      ptr2 = strchr(buf,',');

	      while(ptr2 != NULL) {
		*ptr2 = '\x0';
		zeroesReal.push_back(atof(ptr+1));

		ptr = ptr2;
		ptr2 = strchr(ptr+1,',');
	      }

	      zeroesReal.push_back(atof(ptr+1));	    
	    }

	    if(strstr(buf,"FilterZeroesImag") == buf) {
	      sOption = 1;
	      haveImagZeroes = 1;
	      doFilter = 1;

	      char* ptr2;

	      ptr = strchr(buf,'='); 
	      ptr2 = strchr(buf,',');

	      while(ptr2 != NULL) {
		*ptr2 = '\x0';
		zeroesImag.push_back(atof(ptr+1));

		ptr = ptr2;
		ptr2 = strchr(ptr+1,',');
	      }

	      zeroesImag.push_back(atof(ptr+1));	    
	    }






	    if(strstr(buf,"end;") == buf) {
	      sOption = 1;

	      if(defOptions >= 511) allOptions = 1;
	      else {
		char b[2048];
		sprintf(b,"Missing options in EventsList block starting at line %i of file .tid",sline);
		TidWarning(b);
	      }

	      break;
	    }

	    if(!sOption) {
	      char b[2048];
	      sprintf(b,"Unknown option %s at line %i of file .tid",buf,lnumb);
	      TidWarning(b);
	      Abort = 1;
	      break;
	    }
	  }

	  if(feof(in)!=0) {Stop = 1; break;}

	}

	if(doFilter) {
	  if(!haveRealPoles || !haveImagPoles || !haveRealZeroes || !haveImagZeroes) {
	    TidWarning("One of the FilterZeroesReal, FilterZeroesImag, FilterPolesReal or FilterPolesImag option is missing.");
	    Abort=1;
	  }
	  else {
	    for(unsigned int i=0; i<polesReal.size(); i++)
	      poles.push_back(dComplex(polesReal[i], polesImag[i]));

	    for(unsigned int i=0; i<zeroesReal.size(); i++)
	      zeroes.push_back(dComplex(zeroesReal[i], zeroesImag[i]));
	  }
	}


	if(Abort) {
	  char b[2048];
	  sprintf(b,"Invalid EventsList block starting at line %i of file .tid",sline);
	  TidWarning(b);

	  bool noEnd = 0;

	  for(;;) {
	    fgets(buf, 2048, in);

	    lnumb++;

	    if(feof(in)!=0) {Stop = 1; noEnd = 1; break;}

	    if(buf[0]!='#' && buf[0]!='\n' &&
	       strstr(buf,"end;") == buf) break;
	  }

	  if(noEnd) {
	    char b[2048];
	    sprintf(b,"EventsList block starting at line %i of file .tid has no termination end keyword",sline);
	    TidWarning(b);
	  }
	}

	if(!Abort && defOptions==1023) {
	  try {
	    AddChannel(chnName, sigma, delta, alpha, pthresh, fs, flow, fhigh, dec, tres, stabTime, stabTol, testFreq, chnOSC);

	    if(doFilter) {
	      ChnInfo* p = diag.back();
	      p->SetPoles(poles);
	      p->SetZeroes(zeroes);
	      p->ConstructFilter();
	    }
	  }
	  catch (TidError err) {HandleError(err);}

	}
	
      }

      if(Stop) break;

      if(!knownOption) {
	char b[2048];
	sprintf(b,"Unknown option %s at line %i of file .tid",buf,lnumb);
	TidWarning(b);
      }
    }

      if(feof(in)!=0) break;      

  }
}


void Tid::AddChannel(string chnName,
		     unsigned int sigma,
		     vector<unsigned int>& delta,
		     float alpha,
		     float BPP_,
		     unsigned int fs_,
		     double fLow_,
		     double fHigh_,
		     unsigned int dec_,
		     Interval deltaT_,
		     Interval stabScale_,
		     double stabTol_,
		     vector<unsigned int>& testFreq_,
		     bool OSC_)
{
  int cId = 0;

  list<unsigned int>::iterator p = cidlist.end();

  if(p!=cidlist.begin()) {
    p--;
    cidlist.push_back((*p)+1);
    cId = (*p)+1;
  }
  else {
    cidlist.push_back(0);
    cId = 0;
  }


  ChnInfo* tmp = new ChnInfo(cId,
			     chnName, 
			     fs_,
			     fLow_,
			     fHigh_,
			     dec_,
			     deltaT_,
			     stabScale_,
			     stabTol_,
			     testFreq_,
			     BPP_,
			     sigma, 
			     delta, 
			     alpha);

  if(OSC_) tmp->SetOSC(1);

  diag.push_back(tmp);

}


void Tid::WriteParamsToFile(FILE* out) {

  fprintf(out,"# Tid default configuration file\n#Julien Sylvestre, LIGO-MIT.\n\n");

  fprintf(out,"#Source for data; e.g. LHO_Online, LLO_Online, /export/raid0/data/, etc.\nSource=%s\n\n",sName.c_str());

  fprintf(out,"#Time resolution for all channels (seconds)\nTResolution=%g\n\n", tres.GetSecs());

  fprintf(out,"#Port number for data output\nPort=%i\n\n",IO->GetPort());

  list<ChnInfo *>::iterator p;

  for(p=diag.begin(); p!=diag.end(); p++) {

    fprintf(out,"#Beginning of an 'EventsList' block\nEventsList:\n\n");

    fprintf(out,"#channel name\nChannel=%s\n\n",((*p)->GetName()).c_str());

    fprintf(out,"#lowest frequency to be analyzed (Hz)\nLowFreq=%g\n\n",(*p)->GetfLow());

    fprintf(out,"#highest frequency to be analyzed (use keyword 'Nyquist' or frequency in Hz)\nHighFreq=%g\n\n",(*p)->GetfHigh());

    fprintf(out,"#timescale for background noise to be stable, in seconds\nStabTimeScale=%g\n\n",(double)((*p)->GetStabTimeScale()));

    fprintf(out,"# tolerance on chi-2 test on stability\nStabTol=%g\n\n",(*p)->GetStabTol());

    fprintf(out,"# list of frequencies (Hz) on which to test stability (use keyword 'All' to\n# test all frequencies but DC)\nStabTestFreq=");
    if(!((*p)->GetTestFreq().empty())) {
      fprintf(out,"%g",((*p)->GetTestFreq())[0]/(*p)->GetStabTimeScale());
      for(unsigned int i=1; i<(*p)->GetTestFreq().size(); i++)
	fprintf(out,",%g",((*p)->GetTestFreq())[i]/(*p)->GetStabTimeScale());
    }
    fprintf(out,"\n\n");

    fprintf(out,"#Sigma threshold\nSigma=%u\n\n",(*p)->GetSigma());

    fprintf(out,"#Delta thresholds\nDelta=");
    if(!((*p)->GetDelta().empty())) {
      fprintf(out,"%i",((*p)->GetDelta())[0]);
      for(unsigned int i=1; i<(*p)->GetDelta().size(); i++)
	fprintf(out,",%i",((*p)->GetDelta())[i]);
      fprintf(out,"\n\n");
    }

    fprintf(out,"#Alpha threshold\nAlpha=%g\n\n",(*p)->GetAlpha());

    fprintf(out,"#Black Pixel Probability\nBPP=%g\n\n",(*p)->GetBPP());

    fprintf(out,"#End of 'EventsList' block\nend;\n\n");
  }
}
