/* -*- mode: c++; c-basic-offset: 4; -*- */
#ifndef FRSTATDATAREF_HH
#define FRSTATDATAREF_HH

#include <string>
#ifndef __CINT__
#include "framecpp/Types.hh"
#else
namespace FrameCPP {
    class FrStatData;
}
#endif

class FrVectRef;
class Time;

/**  Class FrStatDataRef contains a wrapped reference to an FrStatData and 
  *  supplies various data accessor methods.
  *  @memo Lib independent FrStatData reference
  *  @author J. Zweizig
  *  @version 1.0; last modified Jul 30, 2006
  */
class FrStatDataRef {

//-------------------------------------  Lib-dependent types
public:
  typedef unsigned long    size_type;
  typedef FrameCPP::FrStatData stat_type;

//-------------------------------------  Class methods
public:
  FrStatDataRef(void);
  FrStatDataRef(const stat_type* x);
  FrStatDataRef(const std::string& name, const std::string& comment, 
		const std::string& repr, long vsn, const Time& tStart, 
		const Time& tEnd, void* det, FrVectRef& v);
  ~FrStatDataRef(void);
  const char* getComment(void) const;
  Time        getEndTime(void) const;
  FrVectRef   getFrVect(void) const;
  const char* getName(void) const;
  const char* getRepresentation(void) const;
  Time        getStartTime(void) const;
  long        getVersion(void) const;
  bool        null(void) const;
  stat_type*  release(void);
  stat_type*  replicate(void) const;
  void        setStat(const std::string& name, const std::string& comment, 
		      const std::string& repr, long vsn, const Time& tStart,
		      const Time& tEnd, void* det, FrVectRef& v);
  void        setStat(stat_type* v, bool owner=false);

private:
  bool       mOwner;
  const stat_type* mData;
};

inline FrStatDataRef::stat_type*
FrStatDataRef::release(void) {
    if (!mOwner) return replicate();
    mOwner = false;
    return const_cast<stat_type*>(mData);
}

inline bool
FrStatDataRef::null(void) const {
    return !mData;
}

#endif // FRVECTREF_HH
